﻿using FluentBlazorApp.Data;
using FluentBlazorApp.DTOs;
using FluentBlazorApp.Models;
using FluentBlazorApp.Responses;
using Microsoft.IdentityModel.Tokens;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;
using static FluentBlazorApp.Responses.CustomResponses;

namespace FluentBlazorApp.Repos
{
    public class Account : IAccount
    {
        private readonly AppDbContext _appDbContext;
        private readonly IConfiguration _config;
        private readonly TimeSpan _expireHour;

        public Account(AppDbContext appDbContext, IConfiguration config)
        {
            _appDbContext = appDbContext;
            _config = config;

            double expireHour = 1d;
            double.TryParse(config["Jwt:ExpireDurationHour"], out expireHour);
            _expireHour = TimeSpan.FromHours(expireHour);
        }


        public async Task<LoginResponse> LoginAsync(LoginDTO model)
        {
            var findUser = await GetUser(model.Name);
            if (findUser == null)
                return new LoginResponse(false, "User does not exist.");

            if (!BCrypt.Net.BCrypt.Verify(model.Password, findUser!.Password))
                return new LoginResponse(false, "Invalid account information.");

            string jwtToken = GenerateToken(findUser);
            return new LoginResponse(true, "Success", jwtToken);
        }

        public async Task<RegistrationResonse> RegisterAsync(RegisterDTO model)
        {
            var findUser = await GetUser(model.Name);
            if (findUser != null)
                return new RegistrationResonse(false, "User already exist.");

            _appDbContext.Users.Add(
                new ApplicationUser()
                {
                    Name = model.Name,
                    Password = BCrypt.Net.BCrypt.HashPassword(model.Password)
                });

            await _appDbContext.SaveChangesAsync();
            return new RegistrationResonse(true, "Success");
        }

        private async Task<ApplicationUser> GetUser(string name)
        {
            return _appDbContext.Users.FirstOrDefault(u => u.Name == name);
        }

        private string GenerateToken(ApplicationUser user)
        {
            var securityKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(_config["Jwt:Key"]));
            var credentials = new SigningCredentials(securityKey, SecurityAlgorithms.HmacSha256);
            var userClaims = new[]
            {
                new Claim(ClaimTypes.Name, user.Name!),
            };

            var token = new JwtSecurityToken(
                issuer: _config["Jwt:Issuer"],
                audience: _config["Jwt:Audience"],
                claims: userClaims,
                expires: DateTime.Now.Add(_expireHour),
                signingCredentials: credentials);

            return new JwtSecurityTokenHandler().WriteToken(token);
        }
    }
}
